@@ -1,15 +1,24 @@
 
+extension.reload_stats = function () {
+    if (this.stats_wnd) {
+        this.stats_wnd.reload_request = true;
+        this.stats_wnd.close();
+    }
+}
+
 extension.view_stats = function () {
-    if (this.stat_wnd) {
-        this.stat_wnd.close();
-        this.stat_wnd = null;
+
+    if (this.stats_wnd) {
+        this.stats_wnd.close();
+        this.stats_wnd = null;
         return;
     }
 
     var wnd = gw_new_window_full(null, true, 'Stats');
     gw_object_set_dragable(wnd);
-    this.stat_wnd = wnd;
+    this.stats_wnd = wnd;
     wnd.extension = this;
+    wnd.reload_request = false;
 
     wnd.area = gw_new_grid_container(wnd);
     wnd.area.break_at_hidden = true;
@@ -20,46 +29,172 @@ extension.view_stats = function () {
     var nb_http = root_odm.nb_http;
     var nb_buffering = 0;
     var nb_ntp_diff = 0;
+    var srd_obj = null;
 
     wnd.has_select = false;
-    wnd.objs = [];
 
-    wnd.gather_res = function (root, selected_service) {
-        //if not dynamic scene, add main OD to satts
-        if (!root.dynamic_scene)
-            this.objs.push(root);
-
-        for (var res_i = 0; res_i < root.nb_resources; res_i++) {
-            var m = root.get_resource(res_i);
-            if (!m) continue;
-            if (m.service_id && (m.service_id != selected_service)) continue;
-
-            if (!m.dynamic_scene) {
-                this.objs.push(m);
-            }
-
-            if (m.type == 'Scene' || m.type == 'Subscene') {
-                this.gather_res(m);
-            }
-        }
-    }
-
-    wnd.gather_res(root_odm, root_odm.selected_service);
-
-    for (var res_i = 0; res_i < wnd.objs.length; res_i++) {
-        var m = wnd.objs[res_i];
+    for (var res_i = 0; res_i < wnd.extension.stats_resources.length; res_i++) {
+        var m = wnd.extension.stats_resources[res_i];
+		var num_qualities;
+		var odm_srd = m.get_srd();
         m.gui = {};
+		if (!srd_obj && odm_srd) srd_obj = m;
+		if (m.dependent_group_id) {
+			srd_obj = m;
+			odm_srd = m.get_srd(m.dependent_group_id);
+		}
 
         var label = '' + m.type;
-        label += ' ID ' + m.ID;
-        if (m.width) label += ' (' + m.width + 'x' + m.height + ')';
+		if (m.dependent_group_id) label += '(Dep. Group)';
+        else if (m.scalable_enhancement) label += ' (Enh. Layer)';
+        else if (m.width) label += ' (' + m.width + 'x' + m.height + ')';
         else if (m.samplerate) label += ' (' + m.samplerate + ' Hz ' + m.channels + ' channels)';
+		
+		if (odm_srd) {
+		 label += ' (SRD ' + odm_srd.x + ',' + odm_srd.y + ',' + odm_srd.w + ',' + odm_srd.h + ')';
+		}
 
         m.gui.txt = gw_new_text(wnd.area, label, 'lefttext');
 
+		num_qualities = m.nb_qualities;
+		if (m.dependent_group_id) {
+			var q = m.get_quality(1, m.dependent_group_id);
+			if (q) num_qualities=2;
+		}
 
+        if (num_qualities > 1) {
+            wnd.has_select = true;
+            m.gui.select_label = gw_new_button(wnd.area, 'Quality');
+            m.gui.select_label.odm = m;
+            m.gui.select_label.on_click = function () {
+                var idx = this.odm.gui.select.value;
+                if (this.odm.gui.selected_idx == idx) {
+                    this.on_long_click();
+                    return;
+                }
+
+                if (idx) {
+                    if (this.odm.gui.qualities[idx - 1].disabled) {
+                        var notif = gw_new_message(null, 'Cannot select representation', 'Representation has been disabled');
+                        notif.set_size(20 * gwskin.default_text_font_size, (2 * gwskin.default_icon_height + gwskin.default_text_font_size));
+                        notif.show();
+                        return;
+                    }
+                    this.odm.select_quality(this.odm.gui.qualities[idx - 1].ID, this.odm.dependent_group_id);
+                } else {
+                    this.odm.select_quality('auto', this.odm.dependent_group_id);
+
+                    this.odm.update_qualities();
+                }
+            }
+            m.gui.select_label.on_long_click = function () {
+                var idx = this.odm.gui.select.value;
+                if (!idx) return;
+                var q = this.odm.gui.qualities[idx - 1];
+
+                var iw = gw_new_window_full(null, true, 'Representation ' + q.ID + ' info');
+                gw_object_set_dragable(iw);
+                iw.area = gw_new_text_area(iw, '');
+
+                var label = 'Mime ' + q.mime + ' - codecs ' + q.codec;
+                label += '\n';
+                label += 'Bandwidth: ' + q.bandwidth;
+                label += '\n';
+                label += 'Enabled: ' + !q.disabled;
+                label += '\n';
+                if (q.width) {
+                    label += 'Size ' + q.width + 'x' + q.height + (q.interlaced ? ' interlaced @' : ' progressive @') + q.fps + ' FPS - SAR ' + q.par_num + '/' + q.par_den;
+                } else if (q.samplerate) {
+                    label += 'Samplerate ' + q.samplerate + ' Hz - ' + q.channels + ' channels';
+                }
+
+                iw.area.set_content(label);
+
+                iw.on_display_size = function (width, height) {
+                    var w = 25 * gwskin.default_text_font_size;
+                    var h = 2 * gwskin.default_icon_height + 5 * gwskin.default_text_font_size;
+                    this.set_size(w, h);
+                }
+
+                iw.on_display_size(gw_display_width, gw_display_height);
+                iw.set_alpha(0.9);
+                iw.show();
+            }
+
+            m.gui.select = gw_new_spincontrol(wnd.area, true);
+            m.gui.select.odm = m;
+            m.gui.select.on_click = function (val) {
+                this.odm.show_select(val);
+            }
+
+
+            m.update_qualities = function () {
+                var sel_idx = 0;
+                this.gui.qualities = [];
+				if (this.dependent_group_id) {
+					var dqidx=0;
+					while (1) {
+						var q = this.get_quality(dqidx, this.dependent_group_id);
+						if (!q) break;
+						this.gui.qualities.push(q);
+						dqidx++;
+					}
+				} else {
+					for (i = 0; i < this.nb_qualities; i++) {
+						var q = this.get_quality(i);
+						if (q) {
+							this.gui.qualities.push(q);
+						}
+					}
+				}
+                this.gui.qualities.sort(function (a, b) { return a.bandwidth - b.bandwidth });
+
+                for (i = 0; i < this.gui.qualities.length; i++) {
+                    if (this.gui.qualities[i].is_selected)
+                        sel_idx = i + 1;
+                }
+
+                this.gui.select.min = 0;
+                this.gui.select.max = this.gui.qualities.length;
+                if (this.gui.selected_idx != sel_idx) {
+                    this.show_select(sel_idx);
+                    this.gui.selected_idx = sel_idx;
+                    this.gui.select.value = sel_idx;
+                }
+            }
+
+            m.show_select = function (idx) {
+                if (!idx) {
+                    this.gui.select_label.set_label('Auto');
+                } else {
+                    var label = '';
+                    var q = this.gui.qualities[idx - 1];
+                    if (!q) return;
+
+                    if (q.is_selected) label += '*';
+
+                    if (q.width) {
+                        label += q.height + (q.interlaced ? 'i' : 'p');
+                    } else if (q.samplerate) {
+                        label += q.samplerate;
+                        if (q.channels) label += '/' + q.channels;
+                    }
+                    label += '@' + Math.round(q.bandwidth / 1000) + 'Kbps';
+
+                    this.gui.select_label.set_label(label);
+                }
+            }
+            m.update_qualities();
+        } else {
+            m.gui.select = null;
+            m.gui.select_label = null;
+            m.update_qualities = function () { }
+        }
+
+		m.gui.info = null;
+		if (!m.dependent_group_id) {
+		
         m.gui.info = gw_new_icon(wnd.area, 'information');
-
         m.gui.info.odm = m;
         m.gui.info.on_click = function () {
             if (this.odm.gui.info_wnd) {
@@ -100,7 +235,7 @@ extension.view_stats = function () {
                     var fps = m.frame_duration;
                     label += 'Size:' + m.width + 'x' + m.height;
                     if (m.pixelformat) label += ' (' + m.par + ' ' + m.pixelformat + ')';
-                } else {
+                } else if (m.samplerate) {
                     label += '' + m.samplerate + 'Hz ' + m.channels + ' channels';
                 }
                 label += '\n'
@@ -125,19 +260,19 @@ extension.view_stats = function () {
                 label += '\n'
                 label += 'Average bitrate: ';
                 bw = m.avg_bitrate;
-                if (bw < 8000000) label += '' + Math.round(bw / 1024) + ' kbps';
-                else label += '' + Math.round(bw / 1024 / 1024) + ' mbps';
+                if (bw < 8000000) label += '' + Math.round(bw / 1000) + ' Kbps';
+                else label += '' + Math.round(bw / 1000 / 1000) + ' Mbps';
 
                 label += ' - Maximum ';
                 bw = m.max_bitrate;
-                if (bw < 8000000) label += '' + Math.round(bw / 1024) + ' kbps';
-                else label += '' + Math.round(bw / 1024 / 1024) + ' mbps';
+                if (bw < 8000000) label += '' + Math.round(bw / 1000) + ' Kbps';
+                else label += '' + Math.round(bw / 1000 / 1000) + ' Mbps';
 
                 label += '\n'
                 label += 'Download bandwidth: ';
                 var bw = m.bandwidth_down;
-                if (bw < 8000000) label += '' + Math.round(bw / 1024) + ' kbps';
-                else label += '' + Math.round(bw / 1024 / 1024) + ' mbps';
+                if (bw < 8000) label += '' + bw + ' Kbps';
+                else label += '' + Math.round(bw / 1000) + ' Mbps';
 
                 label += '\n'
                 label += 'Codec: ' + m.codec;
@@ -161,14 +296,14 @@ extension.view_stats = function () {
             odm.gui.info_wnd.on_display_size(gw_display_width, gw_display_height);
             odm.gui.info_wnd.set_alpha(0.9);
             odm.gui.info_wnd.show();
-
         }
-
+		}
+		
         m.gui.buffer = null;
         m.gui.play = null;
 
         if (m.selected_service == m.service_id) {
-            if (m.max_buffer) {
+            if (m.max_buffer && !m.dependent_group_id) {
                 nb_buffering++;
                 m.gui.buffer = gw_new_gauge(wnd.area, 'Buffer');
             }
@@ -183,139 +318,20 @@ extension.view_stats = function () {
             }
         }
 
-        if (m.nb_qualities > 1) {
-            wnd.has_select = true;
-            m.gui.select_label = gw_new_button(wnd.area, 'Quality');
-            m.gui.select_label.odm = m;
-            m.gui.select_label.on_click = function () {
-                var idx = this.odm.gui.select.value;
-                if (this.odm.gui.selected_idx == idx) {
-                    this.on_long_click();
-                    return;
-                }
-
-                if (idx) {
-                    if (this.odm.gui.qualities[idx - 1].disabled) {
-                        var notif = gw_new_message(null, 'Cannot select representation', 'Representation has been disabled');
-                        notif.set_size(20 * gwskin.default_text_font_size, (2 * gwskin.default_icon_height + gwskin.default_text_font_size));
-                        notif.show();
-                        return;
-                    }
-                    this.odm.select_quality(this.odm.gui.qualities[idx - 1].ID);
-                } else {
-                    this.odm.select_quality('auto');
-
-                    this.odm.update_qualities();
-                }
-            }
-            m.gui.select_label.on_long_click = function () {
-                var idx = this.odm.gui.select.value;
-                if (!idx) return;
-                var q = this.odm.gui.qualities[idx - 1];
-
-                var iw = gw_new_window_full(null, true, 'Representation ' + q.ID + ' info');
-                gw_object_set_dragable(iw);
-                iw.area = gw_new_text_area(iw, '');
-
-                var label = 'Mime ' + q.mime + ' - codecs ' + q.codec;
-                label += '\n';
-                label += 'Bandwidth: ' + q.bandwidth;
-                label += '\n';
-                label += 'Enabled: ' + !q.disabled;
-                label += '\n';
-                if (q.width) {
-                    label += 'Size ' + q.width + 'x' + q.height + (q.interlaced ? ' interlaced @' : ' progressive @') + q.fps + ' FPS - SAR ' + q.par_num + '/' + q.par_den;
-                } else {
-                    label += 'Samplerate ' + q.samplerate + ' Hz - ' + q.channels + ' channels';
-                }
-
-                iw.area.set_content(label);
-
-                iw.on_display_size = function (width, height) {
-                    var w = 25 * gwskin.default_text_font_size;
-                    var h = 2 * gwskin.default_icon_height + 5 * gwskin.default_text_font_size;
-                    this.set_size(w, h);
-                }
-
-                iw.on_display_size(gw_display_width, gw_display_height);
-                iw.set_alpha(0.9);
-                iw.show();
-            }
-
-            m.gui.select = gw_new_spincontrol(wnd.area, true);
-            m.gui.select.odm = m;
-            m.gui.select.on_click = function (val) {
-                this.odm.show_select(val);
-            }
-
-
-            m.update_qualities = function () {
-                var sel_idx = 0;
-                this.gui.qualities = [];
-                for (i = 0; i < this.nb_qualities; i++) {
-                    var q = this.get_quality(i);
-                    if (q) {
-                        this.gui.qualities.push(q);
-                    }
-                }
-                this.gui.qualities.sort(function (a, b) { return a.bandwidth - b.bandwidth });
-
-                for (i = 0; i < this.gui.qualities.length; i++) {
-                    if (this.gui.qualities[i].is_selected)
-                        sel_idx = i + 1;
-                }
-
-                this.gui.select.min = 0;
-                this.gui.select.max = m.nb_qualities;
-                if (this.gui.selected_idx != sel_idx) {
-                    this.show_select(sel_idx);
-                    this.gui.selected_idx = sel_idx;
-                    this.gui.select.value = sel_idx;
-                }
-            }
-
-            m.show_select = function (idx) {
-                if (!idx) {
-                    this.gui.select_label.set_label('Auto');
-                } else {
-                    var label = '';
-                    var q = this.gui.qualities[idx - 1];
-                    if (!q) return;
-
-                    if (q.is_selected) label += '*';
-
-                    if (q.width) {
-                        label += q.height + (q.interlaced ? 'i' : 'p');
-                    } else if (q.samplerate) {
-                        label += q.samplerate;
-                        if (q.channels) label += '/' + q.channels;
-                    }
-                    if (q.bandwidth < 1000000) label += '@' + Math.round(q.bandwidth / 1000) + 'K';
-                    else label += '@' + Math.round(q.bandwidth / 10000) / 100 + 'M';
-
-                    this.gui.select_label.set_label(label);
-                }
-            }
-            m.update_qualities();
-
-        } else {
-            m.gui.select = null;
-            m.gui.select_label = null;
-        }
-
         nb_http += m.nb_http;
 
         gw_new_separator(wnd.area);
     }
 
     wnd.http_control = null;
-    if (nb_http) {
+    if (nb_http || wnd.has_select) {
         wnd.http_text = gw_new_text(wnd.area, 'HTTP rate', 'lefttext');
+
         wnd.http_control = gw_new_slider(wnd.area);
 
         if (gpac.http_max_bitrate) {
-            var br = Math.round(100 * gpac.http_max_bitrate / 1024 / 1024) / 100;
-            if (br > 200) bt = 200;
+            var br = Math.round(100 * gpac.http_max_bitrate / 1000 / 1000) / 100;
+            if (br > 200) br = 200;
             if (br <= 1) {
                 v = 30 * br;
             } else if (br <= 10) {
@@ -327,10 +343,9 @@ extension.view_stats = function () {
             }
             wnd.http_control.set_value(v);
 
-            wnd.http_text.set_label('HTTP cap ' + Math.round(100 * gpac.http_max_bitrate / 1024 / 1024) / 100 + ' Mbps');
-
+            wnd.http_text.set_label('HTTP cap ' + Math.round(gpac.http_max_bitrate / 10000)/100 + ' Mbps');
         } else {
-            wnd.http_control.set_value(0);
+            wnd.http_control.set_value(100);
             wnd.http_text.set_label('HTTP cap off');
         }
 
@@ -346,17 +361,65 @@ extension.view_stats = function () {
             } else {
                 br = 50 + 150 * (val - 80) / 20;
             }
-            if (br) {
-                this.text.set_label('HTTP cap ' + Math.round(100 * br) / 100 + ' Mbps');
-            } else {
+            if (br == 200) {
                 this.text.set_label('HTTP cap off');
+                gpac.http_max_bitrate = Math.round(0);
+            } else if (br == 0) {
+                this.text.set_label('HTTP disable');
+                gpac.http_max_bitrate = -1;
+            } else {
+                this.text.set_label('HTTP cap ' + Math.round(100 * br) / 100 + ' Mbps');
+                gpac.http_max_bitrate = Math.round(br * 1000000);
             }
-            gpac.http_max_bitrate = Math.round(br * 1024 * 1024);
         }
 
         gw_new_separator(wnd.area);
     }
 
+    wnd.srd_modes = null;
+	if (srd_obj) {
+        wnd.srd_text = gw_new_text(wnd.area, 'SRD Control', 'lefttext');
+        wnd.srd_modes = gw_new_button(wnd.area, 'Modes');
+		wnd.srd_modes.wnd = wnd;
+		wnd.srd_modes.srd_obj = srd_obj;
+		wnd.srd_modes.on_click = function() {
+			this.wnd.srd_selected_mode = this.wnd.srd_current_mode;
+			this.wnd.srd_set_mode(this.wnd.srd_current_mode);
+			this.srd_obj.select_quality(this.wnd.srd_selected_mode);
+		}
+		wnd.srd_select = gw_new_spincontrol(wnd.area, true);
+		wnd.srd_select.wnd = wnd;
+		wnd.srd_select.on_click = function (val) {
+			this.wnd.srd_set_mode(val);
+		}
+        wnd.srd_select.min = 0;
+        wnd.srd_select.max = 8;
+        wnd.srd_selected_mode = 0;
+		if (srd_obj.gui.qualities != null)
+			srd_obj.gui.qualities[0].tile_mode;
+        wnd.srd_current_mode = 0;
+		
+		wnd.srd_set_mode = function(mode) {
+		 var label = '';
+		 this.srd_current_mode = mode;
+		 if (this.srd_current_mode == this.srd_selected_mode) label='* ';
+
+		 switch (mode) {
+		 case 0: label+='Equal'; break;
+		 case 1: label+='Top Rows'; break;
+		 case 2: label+='Bottom Rows'; break;
+		 case 3: label+='Middle Rows'; break;
+		 case 4: label+='Left Column'; break;
+		 case 5: label+='Right Column'; break;
+		 case 6: label+='Middle Column'; break;
+		 case 7: label+='Center'; break;
+		 case 8: label+='Edges'; break;
+		 }
+		 this.srd_modes.set_label(label);
+		}
+		wnd.srd_set_mode(wnd.srd_selected_mode);
+	}
+
     wnd.plot = gw_new_plotter(wnd.area);
 
 
@@ -367,32 +430,34 @@ extension.view_stats = function () {
         w = 20 * gwskin.default_text_font_size;
         w += 4 * gwskin.default_icon_height;
         if (this.has_select)
-            w += 4 * gwskin.default_icon_height;
+            w += 6 * gwskin.default_icon_height;
 
-        for (var i = 0; i < this.objs.length; i++) {
+        for (var i = 0; i < this.extension.stats_resources.length; i++) {
+            var res = this.extension.stats_resources[i];
             var aw = w;
 
-            this.objs[i].gui.info.set_size(1.5 * gwskin.default_icon_height, gwskin.default_icon_height);
-            aw -= this.objs[i].gui.info.width;
-
-            if (this.objs[i].gui.buffer) {
-                this.objs[i].gui.buffer.set_size(3 * gwskin.default_icon_height, 0.75 * gwskin.default_icon_height);
+            if (res.gui.info) {
+				res.gui.info.set_size(1.5 * gwskin.default_icon_height, gwskin.default_icon_height);
+				aw -= res.gui.info.width;
+			}
+            if (res.gui.buffer) {
+                res.gui.buffer.set_size(2 * gwskin.default_icon_height, 0.75 * gwskin.default_icon_height);
             }
-            if (this.objs[i].gui.play) {
-                this.objs[i].gui.play.set_size(gwskin.default_icon_height, gwskin.default_icon_height);
+            if (res.gui.play) {
+                res.gui.play.set_size(gwskin.default_icon_height, gwskin.default_icon_height);
             }
             aw -= 4 * gwskin.default_icon_height;
 
-            if (this.objs[i].gui.select) {
-                this.objs[i].gui.select.set_size(gwskin.default_icon_height, gwskin.default_icon_height);
+            if (res.gui.select) {
+                res.gui.select.set_size(gwskin.default_icon_height, gwskin.default_icon_height);
                 aw -= gwskin.default_icon_height;
-                this.objs[i].gui.select_label.set_size(4 * gwskin.default_icon_height, 0.75 * gwskin.default_icon_height);
+                res.gui.select_label.set_size(6 * gwskin.default_icon_height, 0.75 * gwskin.default_icon_height);
                 aw -= 4 * gwskin.default_icon_height;
             }
 
 
-            this.objs[i].gui.txt.set_size(aw, gwskin.default_icon_height);
-            this.objs[i].gui.txt.set_width(aw);
+            res.gui.txt.set_size(aw, gwskin.default_icon_height);
+            res.gui.txt.set_width(aw);
             h += gwskin.default_icon_height;
         }
 
@@ -401,49 +466,58 @@ extension.view_stats = function () {
             h += 8 * gwskin.default_icon_height;
         }
 
-        if (wnd.http_control) {
-            wnd.http_text.set_size(11 * gwskin.default_text_font_size, gwskin.default_icon_height);
-            wnd.http_control.set_size(w - 11 * gwskin.default_text_font_size, 0.2 * gwskin.default_icon_height);
+        if (this.http_control) {
+            this.http_text.set_size(11 * gwskin.default_text_font_size, gwskin.default_icon_height);
+            var ch = 0.5 * gwskin.default_icon_height;
+            this.http_control.set_size(w - 11 * gwskin.default_text_font_size, ch, ch, ch);
             h += gwskin.default_icon_height;
         }
+		
+		if (this.srd_modes) {
+			var cw = 0.75 * gwskin.default_icon_height;
+			this.srd_text.set_size(12 * gwskin.default_text_font_size, gwskin.default_icon_height);
+			this.srd_modes.set_size(w - 12 * gwskin.default_text_font_size - gwskin.default_icon_height, cw);
+			this.srd_select.set_size(gwskin.default_icon_height, cw);
+		}
 
         this.set_size(w, h);
         this.move((w - width) / 2, (height - h) / 2);
     }
     wnd.on_close = function () {
-        this.timer.stop();
-        this.objs = [];
-        this.extension.stat_wnd = null;
+        //this.timer.stop();
+        //this.objs = [];
+        if (this.reload_request) {
+            var translation = this.extension.stats_wnd.translation;
+            this.extension.stats_wnd = null;
+            this.extension.view_stats();
+            this.extension.stats_wnd.translation = translation;
+        } else {
+            this.extension.stats_wnd = null;
+        }
     }
 
-    wnd.timer = gw_new_timer(false);
-    wnd.timer.wnd = wnd;
-    wnd.timer.set_timeout(0.25, true);
-
     var label = 'Statistics (' + gpac.nb_cores + ' cores - ';
-    if (gpac.system_memory > 1000000000) label += '' + Math.round(gpac.system_memory / 1024 / 1024 / 1024) + ' GB RAM)';
-    else label += '' + Math.round(gpac.system_memory / 1024 / 1024) + ' MB RAM)';
+    if (gpac.system_memory > 1000000000) label += '' + Math.round(gpac.system_memory / 1000 / 1000 / 1000) + ' GB RAM)';
+    else label += '' + Math.round(gpac.system_memory / 1000 / 1000) + ' MB RAM)';
 
     wnd.set_label(label);
 
-    wnd.stats = [];
-    wnd.stats_window = 32;
-
     if (wnd.plot) {
         wnd.s_fps = wnd.plot.add_serie('FPS', 'Hz', 0.8, 0, 0);
         if (nb_http) {
-            wnd.s_bw = wnd.plot.add_serie('BW', 'Mbps', 0.8, 0, 0.8);
+            wnd.s_bw = wnd.plot.add_serie('BW', 'Kbps', 0.8, 0, 0.8);
         } else {
             wnd.s_bw = null;
         }
-        wnd.s_bitrate = wnd.plot.add_serie('Rate', 'Mbps', 0, 0.8, 0);
+        wnd.s_bitrate = wnd.plot.add_serie('Rate', 'Kbps', 0, 0.8, 0);
+        wnd.s_quality = wnd.plot.add_serie('Quality', 'Kbps', 1, 0.65, 0);
         if (nb_buffering)
-            wnd.s_buf = wnd.plot.add_serie('Buffer', 's', 0, 0, 0.8);
+            wnd.s_buf = wnd.plot.add_serie('Buffer', 'ms', 0, 0, 0.8);
         else
             wnd.s_buf = null;
 
         if (nb_ntp_diff)
-            wnd.s_ntp = wnd.plot.add_serie('NTP diff', 'ms', 0, 0.3, 0.8);
+            wnd.s_ntp = wnd.plot.add_serie('E2E delay', 'ms', 0, 0.3, 0.8);
         else
             wnd.s_ntp = null;
 
@@ -452,122 +526,83 @@ extension.view_stats = function () {
 
     }
 
-    wnd.timer.on_event = function (val) {
-        var wnd = this.wnd;
-        var nb_buff = 0;
-       
-        var stat_obj = null;
-        //stats every second
-        if (!(val % 4)) {
-            if (wnd.stats.length >= wnd.stats_window) {
-                wnd.stats.splice(0, 1);
-            }
-            stat_obj = {};
-            wnd.stats.push(stat_obj);
+    wnd.update_series = function() {
+        var ext = this.extension;
+        var length = (ext.stats_data.length > ext.stats_window ? stats_window : ext.stats_data.length);
 
-            var t = new Date()
-            stat_obj.time = Math.round(t.getTime() / 1000);
-            stat_obj.fps = Math.round(100 * gpac.fps) / 100;
-            stat_obj.cpu = gpac.cpu;
-            stat_obj.memory = Math.round(100 * gpac.memory / 1024 / 1024) / 100;
-            stat_obj.bitrate = 0;
-            if (wnd.s_bw) {
-                var b = gpac.http_bitrate / 1024 / 1024;
-                stat_obj.http_bandwidth = Math.round(100 * b) / 100;
-            }
-            stat_obj.buffer = 0;
-            stat_obj.cumulated_bandwidth = 0;
-            stat_obj.ntp_diff = 0;
+        this.s_fps.refresh_serie(ext.stats_data, 'time', 'fps', length, 4);
+        if (this.s_bw) {
+            this.s_bw.refresh_serie(ext.stats_data, 'time', 'http_bandwidth', length, 1);
         }
-
-        for (var i = 0; i < wnd.objs.length; i++) {
-            var m = wnd.objs[i];
-            
-            if (m.gui.buffer) {
-                var label = ' ' + m.type;
-                label += ' ID ' + m.ID;
-                if (m.width) label += ' (' + m.width + 'x' + m.height + ')';
-                else if (m.samplerate) label += ' (' + m.samplerate + ' Hz ' + m.channels + ' channels)';
-                m.gui.txt.set_label(label);
-
-                var bl;
-                if (m.max_buffer) {
-                    var speed = wnd.extension.movie_control.mediaSpeed;
-                    if (speed < 0) speed = 1;
-                    else if (speed == 0) speed = 1;
-                    var buf = m.buffer / speed;
-                    bl = 100 * buf / m.max_buffer;
-
-                    if (stat_obj) {
-//                        if (stat_obj.buffer < buf) {
-//                            stat_obj.buffer = buf;
-//                        }
-
-                        stat_obj.buffer += buf;
-                        nb_buff++;
-                    }
-                }
-                else bl = 100;
-                m.gui.buffer.set_value(bl);
-
-                m.gui.buffer.set_label('' + Math.round(m.buffer / 10) / 100 + ' s');
-
-                if (stat_obj) {
-                  bl = m.ntp_diff;
-                  if (bl > stat_obj.ntp_diff) 
-                    stat_obj.ntp_diff = bl;
-                }
-            }
-
-            if (stat_obj) {
-                stat_obj.bitrate += Math.round(100 * m.avg_bitrate / 1024 / 1024) / 100;
-                stat_obj.cumulated_bandwidth += m.bandwidth_down;
-            }
+        if (this.s_buf) {
+            this.s_buf.refresh_serie(ext.stats_data, 'time', 'buffer', length, 1.5);
         }
-
-        if (stat_obj && wnd.stats.length) {
-            wnd.s_fps.refresh_serie(wnd.stats, 'time', 'fps', wnd.stats_window, 4);
-            if (wnd.s_bw) {
-                wnd.s_bw.refresh_serie(this.wnd.stats, 'time', 'http_bandwidth', wnd.stats_window, 1);
-            }
-            if (wnd.s_buf) {
-                if (nb_buff) {
-                    stat_obj.buffer /= nb_buff;
-                }
-                wnd.s_buf.refresh_serie(this.wnd.stats, 'time', 'buffer', wnd.stats_window, 1.5);
-            }
-            if (stat_obj.bitrate) {
-                wnd.s_bitrate.refresh_serie(this.wnd.stats, 'time', 'bitrate', wnd.stats_window, 2);
-            } else {
-                wnd.s_bitrate.hide();
-            }
-            if (wnd.s_ntp) {
-                wnd.s_ntp.refresh_serie(this.wnd.stats, 'time', 'ntp_diff', wnd.stats_window, 3);
-            }
-            
-            wnd.s_cpu.refresh_serie(wnd.stats, 'time', 'cpu', wnd.stats_window, 10);
-            wnd.s_mem.refresh_serie(wnd.stats, 'time', 'memory', wnd.stats_window, 6);
+        var stat_obj = ext.stats_data[ext.stats_data.length-1];
+        if (stat_obj.bitrate) {
+            this.s_bitrate.refresh_serie(ext.stats_data, 'time', 'bitrate', length, 2);
         }
+        if (this.s_ntp) {
+            this.s_ntp.refresh_serie(ext.stats_data, 'time', 'ntp_diff', length, 3);
+        }
+        this.s_cpu.refresh_serie(ext.stats_data, 'time', 'cpu', length, 10);
+        this.s_mem.refresh_serie(ext.stats_data, 'time', 'memory', length, 6);
+        this.s_quality.refresh_serie(ext.stats_data, 'time', 'quality', length, 2.5);
     }
 
     wnd.quality_changed = function () {
-        for (var i = 0; i < this.objs.length; i++) {
-            var m = this.objs[i];
+        for (var i = 0; i < this.extension.stats_resources.length; i++) {
+            var m = this.extension.stats_resources[i];
             m.update_qualities();
         }
     }
     wnd.close_all = function () {
-        for (var i = 0; i < this.objs.length; i++) {
-            if (this.objs[i].gui.info_wnd) {
-                this.objs[i].gui.info_wnd.odm = null;
-                this.objs[i].gui.info_wnd.close();
+        for (var i = 0; i < this.extension.stats_resources.length; i++) {
+            if (this.extension.stats_resources[i].gui.info_wnd) {
+                this.extension.stats_resources[i].gui.info_wnd.odm = null;
+                this.extension.stats_resources[i].gui.info_wnd.close();
             }
         }
-        this.objs.length = 0;
+        this.extension.stats_resources.length = 0;
         this.close();
     }
+
+    wnd.update_resource_gui = function(m, bl) {
+        if (m.gui && m.gui.buffer) {
+			var odm_srd = m.get_srd();
+
+			if (m.dependent_group_id) {
+				odm_srd = m.get_srd(m.dependent_group_id);
+			}
+
+            var label = ' ' + m.type;
+
+			if (m.dependent_group_id) label += '(Dep. Group)';
+			else if (m.scalable_enhancement) label += ' (Enh. Layer)';
+            else if (m.width) label += ' (' + m.width + 'x' + m.height + ')';
+            else if (m.samplerate) label += ' (' + Math.round(m.samplerate / 10) / 100 + ' kHz ' + m.channels + ' ch)';
+            else if (m.scalable_enhancement) label += ' (Enh. Layer)';
+
+			if (odm_srd) {
+				label += ' (SRD ' + odm_srd.x + ',' + odm_srd.y + ',' + odm_srd.w + ',' + odm_srd.h + ')';
+			}
+
+            var url = m.service_url;
+/*
+            if ((url.indexOf('udp://') >= 0) || (url.indexOf('rtp://') >= 0) || (url.indexOf('dvb://') >= 0))
+                label += ' Broadcast';
+            else if ((url.indexOf('file://') >= 0) || (url.indexOf('://') < 0))
+                label += ' File';
+            else
+                label += ' Broadband';
+*/
+            m.gui.txt.set_label(label);
+            m.gui.buffer.set_value(bl);
+            m.gui.buffer.set_label('' + Math.round(m.buffer / 10) / 100 + ' s');
+        }
+    }
+
     wnd.on_display_size(gw_display_width, gw_display_height);
     wnd.set_alpha(0.9);
     wnd.show();
 };
-    
\ No newline at end of file
+    
